
/* Chrysalide - Outil d'analyse de fichiers binaires
 * core.c - intégration du support du format Dex
 *
 * Copyright (C) 2017 Cyrille Bagard
 *
 *  This file is part of Chrysalide.
 *
 *  Chrysalide is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Chrysalide is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Chrysalide.  If not, see <http://www.gnu.org/licenses/>.
 */


#include "core.h"


#include <core/global.h>
#include <plugins/plugin-def.h>


#include "format.h"
#include "python/module.h"



DEFINE_CHRYSALIDE_PLUGIN("dex", "Add support for the DEX format", "0.1.0",
                         RL("PyChrysalide", "dexbnf"), AL(PGA_PLUGIN_INIT, PGA_CONTENT_RESOLVER));



/******************************************************************************
*                                                                             *
*  Paramètres  : plugin = greffon à manipuler.                                *
*                                                                             *
*  Description : Prend acte du chargement du greffon.                         *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

G_MODULE_EXPORT bool chrysalide_plugin_init(GPluginModule *plugin)
{
    bool result;                            /* Bilan à retourner           */

    result = register_format_loader("dex", "Dalvik Executable format", g_dex_format_new);

    if (result)
        result = add_format_dex_module_to_python_module();

    return result;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : plugin  = greffon à manipuler.                               *
*                action  = type d'action attendue.                            *
*                content = contenu binaire à traiter.                         *
*                gid     = identifiant du groupe de traitement.               *
*                status  = barre de statut à tenir informée.                  *
*                                                                             *
*  Description : Procède à une opération liée à un contenu binaire.           *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

G_MODULE_EXPORT void chrysalide_plugin_handle_binary_content(const GPluginModule *plugin, PluginAction action, GBinContent *content, gid_t gid, GtkStatusStack *status)
{
    vmpa2t addr;                            /* Tête de lecture initiale    */
    bool test;                              /* Bilan des accès mémoire     */
    char magic[DEX_FILE_MAGIC_LEN];         /* Idenfiant standard          */
    GExeFormat *format;                     /* Format DEX reconnu          */
    GLoadedContent *loaded;                 /* Représentation chargée      */
    GContentResolver *resolver;             /* Resolveur de contenus       */

    init_vmpa(&addr, 0, VMPA_NO_VIRTUAL);

    test = g_binary_content_read_raw(content, &addr, DEX_FILE_MAGIC_LEN, (bin_t *)magic);

    if (test)
        test = (memcmp(magic, DEX_FILE_MAGIC, DEX_FILE_MAGIC_LEN) == 0);

    if (test)
    {
        format = g_dex_format_new(content);
        loaded = g_loaded_binary_new(format);

        resolver = get_current_content_resolver();
        g_content_resolver_add_detected(resolver, gid, loaded);
        g_object_unref(G_OBJECT(resolver));

    }

}
